%% Demo for comparison of the following algorithms:
%% CDBM3D (BM1);
%% Iterative CDBM3D (BM2);
%% SPINPHASE (Sp);
%% Windowed Fourier Transform (W)
%% Paper submited to IEEE Trans on Image Processing
%% 'Complex domain sparse phase imaging based on nonlocal BM3D techniques,'
%%  Vladimir Katkovnik and Karen Egiazarian, June, 2016

%% Vladimir Katkovnik, Tampere, TUT, June, 2016
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clear all
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Selection of tested algorithms: CDBM3D and Iteratice CDBM3D are always on

if_lena=0;             %% !!! if if_lena=1 the algorithm parameters are changed for cameraman/lena images

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if_ITERATIVE_SPAR=0;   %% !!! Disabled in this Demo. Always if_ITERATIVE_SPAR=0 
if_SpInPHASE=1;        %% Index to include SpInPHASE test, using if_SpInPHASE=1
if_KEMAO=1;            %% Index to include WFT test, using if_KEMAO=1
if_CDBM3D=1;           %% Index to include CD-BM3D test, using if_CDBM3D=1
sigma_set=[.1 .3 .5 .7 .9 ];        %% Set of sigma values for the noise
if_figure_on=1;        %% To display figures
if_unwrapping=1;       %% if_unwrapping=1 interferometric phase is unwrapped

 %% The examples 1,3,4, 8, 9, 10,11,12 require the phase unwrapping !!!!
quadratic_phase=0;     %% !!! Do not change it

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% OBJECT IMAGE and NOISE MODELLING  %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% OBJECT PHASE SELECTION %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
test_phase =10;                        
switch(test_phase)
   case 0
  varphi=zeros(100);
  varphi(1:end,50:end)=pi;  
  PhaseName='Invariant phase';

   case 1
load DEMO_1                               %% Gaussian abs phase
PhaseName='DEMO_1';
varphi=varphi;
    case 2
varphi = fspecial('gaussian',[100 100],10);    %% Two peaks
y1=circshift(varphi,[-10,+10]); y2=circshift(varphi,[20,-20]);
varphi=y1-y2;                                 %% True absolute phase

varphi=varphi/max(varphi(:)); y_scale=2*pi*7;
varphi=varphi*y_scale;
[yN,xN]=size(varphi);
PhaseName='Two peaks';
   case 3
load DEMO_3                      %% Truncated Gaussian abs phase
varphi=varphi;
PhaseName='DEMO_3';
   case 4
load DEMO_4_small                 %% Shear plane abs phase
varphi=varphi;
PhaseName='DEMO_4';
%      case 5
% load spiral.surf.mat               %%  Spiral image
% varphi=spiral;
% varphi=varphi(1:256,1:256);
   case 6
        
varphi=double(imread('image_Lena256.png'))/255*2*pi/4;
% varphi=imresize(varphi,[100 100]);
% varphi=varphi(70+(1:128),80+(1:128));
PhaseName='Lena128';
   case 7 
     %   varphi=double(imread('image_Cheese128.png'))/255*2*pi/4;
varphi=double(imread('image_Cameraman256.png'))/255*2*pi/4;
%% varphi=varphi(70-40+[1:128],80+[1:128]);
PhaseName='Cameraman128';

    case 8                            %% Continuous Sinusoidal Surface  (EXP 2)
 PhaseName='ContSin';                
 M=100; N=100;
[X,Y] = meshgrid(-round(N/2): -round(N/2)+N-1, ...
                    -round(M/2): -round(M/2)+M-1);
varphi = 6*sin(2*pi*(X/M+Y/M)*6);
   
    case 9                            %% Discontinuous Sinusoidal Surface  (EXP 3)
PhaseName='DiscontSin';
M=100; N=100;
[X,Y] = meshgrid(-round(N/2): -round(N/2)+N-1, ...
                    -round(M/2): -round(M/2)+M-1);
       
  mask = Y < 0;                
 varphi = 6*sin(2*pi*(X/M)*6).*mask -  6*sin(2*pi*(X/M)*6).*(1-mask);
    case 10                                     %%  Mountains  (EXP 4)
PhaseName='Mountains';
M = 100; N = 100;

S=2055615866; randn('seed', S);  
varphi=randn(M,N);
fc = 0.01; % define cut frequency
mask = zeros(M,N);
mask(round(M/2-fc*M/2):round(M/2+fc*M/2), ...
    round(N/2-fc*N/2):round(N/2+fc*N/2)) = 1;
varphi =  2400* real(ifft2(fftshift(fftshift(fft2(varphi)).*mask)));

    case 11  %%  lena plus quadratic phase
PhaseName='Lena & quadratic phase';
M=128; N=128;
[X,Y] = meshgrid(-round(N/2): -round(N/2)+N-1, ...
                    -round(M/2): -round(M/2)+M-1);
     
quadratic_phase=-(X.^2+Y.^2)/(4*(M/2)^2);    %% Quadratic phase     
quadratic_phase=quadratic_phase+max(abs(quadratic_phase(:)));    
quadratic_phase=quadratic_phase*50; 
varphi=double(imread('image_Lena256.png'))/255*2*pi/4;
varphi=varphi(70+[1:128],80+[1:128]);
varphi=varphi+quadratic_phase;
         
     case 12  %%  camearaman plus quadratic phase
PhaseName='Cameraman & quadratic phase';
M=128; N=128;
[X,Y] = meshgrid(-round(N/2): -round(N/2)+N-1, ...
                    -round(M/2): -round(M/2)+M-1);
 quadratic_phase=-(X.^2+Y.^2)/(4*(M/2)^2);    %% Quadratic Phase     
 quadratic_phase=quadratic_phase+max(abs(quadratic_phase(:)));    
 quadratic_phase=quadratic_phase*100; 
varphi=double(imread('image_Cameraman256.png'))/255*2*pi/4;
varphi=varphi(70-40+[1:128],80+[1:128]);
varphi=varphi+quadratic_phase;
        
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% AMPLITUDE IMAGE SELECTION %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

test_amplitude=1;                        
switch(test_amplitude)

   case 1
B=ones(size(varphi));                      %% Invariant amplitude
   case 2                                  
   M=100; N=100;                           %% Quadratic Amplitude
[X,Y] = meshgrid(-round(N/2): -round(N/2)+N-1, ...
                    -round(M/2): -round(M/2)+M-1);
     
B=-(X.^2+Y.^2)/(4*(M/2)^2)+1.5;    %% Quadratic Amplitude
    case 3
B=abs(varphi/max(varphi(:))+.1);  %% varying amplitude- phase leakage    
   case 4
B=cos(varphi)+.1;                  %% varying amplitude- phase leakage  
      case 6
B=double(imread('image_Lena256.png'))/255;
B=B(70+(1:128),80+(1:128))+.1;
         %% B='Lena256';
    case 7
B=double(imread('image_Cameraman256.png'))/255;
B=B(70-40+[1:128],80+[1:128])+.1;

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% MAIN PROGRAM %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s_sigma=0;
for sigma=sigma_set               %% Loop over sigma of noise
    s_sigma=s_sigma+1
    sigma0=sigma;
%%%%%%%% OBSERVATION MODELLING %%%%%%%%%%%
     disp(['sigma =', num2str(sigma)]);
    
allsigma = sigma;
sigma1= allsigma/sqrt(2);   %% sigma for real and imaginary parts of noise
sigma = 255*sigma1;         %% sigma normalized for CDBM3D
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Y=B.*exp(1i*varphi);                       %% True signal

%%%%%%%%% Noisy Gaussian Data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
S=2055615866; randn('seed', S);

Z=Y + (sigma/255)*(randn(size(B))+1i*randn(size(B))); %% Noisy signal

phi=angle(Z);                               %% Noisy phase phase
%keyboard
[M,N]=size(Z);

SNR(s_sigma)=10*log10(mean(B(:).^2)/(allsigma^2));     %% SNR for observations

%% disp(['SNR =', num2str(SNR),' dB']);

if if_CDBM3D==1
    display('CDBM3D')
   tic
  lambda3D=1.8;     %% threshold parameter for 3D_HOSVD
      
  if if_lena==1
  
  lambda3D=1.1;   %%  for LENA  %% threshold parameter for 3_HOSVD
  
 %%  lambda3D=.9;   %%  for LENA  %% threshold parameter for 3_HOSVD

   end
 %% lambda3D=1;
 if (if_lena==0 )&(sigma0==.1)     %
 lambda3D=2.7;
  end
  %%  lambda3D=6;     %% threshold parameter for 2_HOSVD
%% parameters for CDBM3D %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
param.nm1 = 8;  %% N1 x N1 is the block size used for the hard-thresholding (HT) filtering
param.Nstep =3; %% sliding step to process every next reference block
param.nm2 =floor(param.nm1*param.nm1/2); %% maximum number of similar blocks (maximum size of the 3rd dimension of a 3D array)
param.Ns = 39*2;  %% length of the side of the search neighborhood for full-search block-matching (BM), must be odd

param.lambda3D=lambda3D;         %% parameter for the thresholding
param.lambda_conf_int=2;         %% parameter for the two sides symmetric confidence interval for grouping
param.lambda_conf_int_factor=10; %% parameter for the upper bound in the two sides non-symmetric confidence interval for grouping
param.doBlockMatching=1;         %% (1) means that the new grouping is produced, (0) means that the previous grouping is used 
param.bm2D3Dhosvd_index3D=1;     %% 1 stays for high-order 3D SVD, otherwise 2D SVD
param.sigma = sigma;             %% STD of the additive Gaussian noise
param.lambda2D       =  0.0;     %% threshold parameter for the coarse initial denoising used in the d-distance measure
param.thresholding_type='h';
%% two-sided confidence interval for grouping of similar patches
%lower bound
param.lambda_conf(1)=max(0,(param.nm1^2-param.lambda_conf_int*sqrt(2*param.nm1^2))*param.sigma^2*2/255^2);
%upper bound
param.lambda_conf(2)= (param.nm1^2+param.lambda_conf_int*sqrt(2*param.nm1^2))*param.sigma^2*2/255^2*param.lambda_conf_int_factor;
   
 [CDBM3D_hat]  = CDBM3Dp(Z, param);


%% Phase unwrap  %%%%%%%%%
    XX=CDBM3D_hat;  PUMA_SCRIPT;
    toc_CDBM3D=toc;
%%%%%%%%%%%%%%%%%%%%%%%%%
   
 %%%%%%%%%% CRITERIA EVALUATION %%%%%%%%%%%%


[PSNR_PHASE, RMSE_abs_phase, ISNR_PHASE,ISNR_exp, PSNR_a_PHASE, NELP_PHASE, PSNR_AMPL, ISNR_AMPL]=Accuracy_Criteria_Complex(Z,varphi,B,CDBM3D_hat,unwrap_phase);
%%%%%%%%%%%%%%%%
PSNR_PHASE_CDBM3D_set(s_sigma)=PSNR_PHASE(end)
RMSE_abs_phase_CDBM3D_set(s_sigma)=RMSE_abs_phase(end); %#ok<*SAGROW>
ISNR_PHASE_CDBM3D_set(s_sigma)=ISNR_PHASE;
ISNR_exp_CDBM3D_set(s_sigma)=ISNR_exp;
PSNR_a_PHASE_CDBM3D_set(s_sigma)=PSNR_a_PHASE;
NELP_PHASE_CDBM3D_set(s_sigma)=NELP_PHASE;
PSNR_AMPL_CDBM3D_set(s_sigma)=PSNR_AMPL(end);
ISNR_AMPL_CDBM3D_set(s_sigma)=ISNR_AMPL;
ttime_CDBM3D_set(s_sigma)=toc_CDBM3D;



%%%%%%%%%%%%%%%%%
if if_figure_on==1
figure, subplot(2,2,1), imshow((unwrap_phase- quadratic_phase),[]), title(['CDBM3D ABS PHASE, RMSE = ', num2str(RMSE_abs_phase(end),3), ' dB']),...
subplot(2,2,2), imshow(abs(CDBM3D_hat),[]), title(['AMPL, PSNR = ', num2str(PSNR_AMPL(end),3), ' dB']),...
subplot(2,2,3), imshow((wrap(unwrap_phase- quadratic_phase)),[]), title(['WRAP PHASE, PSNR = ', num2str(PSNR_PHASE(end),3), ' dB']),...
subplot(2,2,4), imshow(wrap((unwrap_phase- quadratic_phase-varphi)),[]), title(['WRAP PHASE ERRORS, \sigma= ',num2str(sigma0) ])
end


    
end  %% for CDBM3D 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% ITERATIVE CDBM3D %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% LOOP OVER THRESHOLDING PARAMETER: lambda3D %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% INITIALIZATION %%%%%%%%%

 u=Z;
vv1=sum((abs(u(:))-B(:)).^2);  PSNR_AMPL1(s_sigma,1)=10*log10(M*N*max(abs(B(:)).^2)/vv1);
vv1=norm(wrap(angle(u) - varphi),'fro')^2; PSNR_PHASE1(s_sigma,1)=10*log10(4*M*N*pi^2/vv1);
sigma_norm=sigma/255;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% set_lambda3D=4. for 2D SVD; %%[.5 .75 .9 1 1.25 ]; %% set of thresholding parameter for CDBM3D
  %% for 3D HOSVD
if sigma0==0.1
    set_lambda3D=2.;
else
    set_lambda3D=1;
end
if if_lena==1
 set_lambda3D=.7;   %%  for LENA  %% threshold parameter for 3D_HOSVD
 
 %% set_lambda3D=.6;   %%  for LENA-cameraman %% threshold parameter for 3D_HOSVD
end
%%set_lambda3D=7.5
s_lambda3D=0;
for lambda3D=set_lambda3D
   tic
lambda3D;
s_lambda3D=s_lambda3D+1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Parameters for CDBM3D %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
param.nm1 = 8;  %% N1 x N1 is the block size used for the hard-thresholding (HT) filtering
param.Nstep =3; %% sliding step to process every next reference block
param.nm2 =floor(param.nm1*param.nm1/2); %% maximum number of similar blocks (maximum size of the 3rd dimension of a 3D array)

if if_lena==1
param.Ns =39 ;  %% length of the side of the search neighborhood for full-search block-matching (BM), must be odd
else
    param.Ns =39*2 ;  %% length of the side of the search neighborhood for full-search block-matching (BM), must be odd
end 
param.lambda3D=lambda3D;       %% parameter for the thresholding
param.lambda_conf_int=2;         %% parameter for the two sides symmetric confidence interval for grouping
param.lambda_conf_int_factor=10; %% parameter for the upper bound in the two sides non-symmetric confidence interval for grouping
param.doBlockMatching=1;         %% (1) means that the new grouping is produced, (0) means that the previous grouping is used 
param.bm2D3Dhosvd_index3D=1;     %% 1 stays for high-order 3D SVD, otherwise 2D SVD
param.sigma = sigma;             %% STD of the additive Gaussian noise
param.lambda2D       =  0.0;     %% threshold parameter for the coarse initial denoising used in the d-distance measure
param.thresholding_type='h';
%lower bound
param.lambda_conf(1)=max(0,(param.nm1^2-param.lambda_conf_int*sqrt(2*param.nm1^2))*param.sigma^2*2/255^2);
%upper bound
param.lambda_conf(2)= (param.nm1^2+param.lambda_conf_int*sqrt(2*param.nm1^2))*param.sigma^2*2/255^2*param.lambda_conf_int_factor;


sigma_norm=sigma/255;

%%%%%%%%%%%%%%%%%%%
%% Parameter Change for Iterative CDBM3D %%%%%%%
param.iterNumber=20/4;

if sigma0<0.5
 param.alpha=1/(1+1/1.5);
  
else
param.alpha=1/(1+1/1.3);
end
  
   param.a1=1;
   param.a2=param.a1;
if if_lena==1
  
   param.alpha=.65;
  
  
   
   param.a1=1/(1+.01);
      
   param.a2=param.a1;
else
%    param.alpha=.65;
%    param.a1=1/(1+.01);
%    param.a2=param.a1;
   
end



%% ITERATIVE CDBM3D %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


[Iterative_CDBM3D_hat, PSNR_PHASE,PSNR_AMPL]  = ITERATIVE_CDBM3Dp(Z,u,param,B,varphi);


if if_figure_on==1
  figure, subplot(1,2,1), plot(squeeze(PSNR_PHASE),'-o'), title('CDBM3D ITER, PSNR_{phase}'),  grid on,xlabel('Iter. num.'), ylabel('dB'),...
        subplot(1,2,2), plot(squeeze(PSNR_AMPL),'-o'), title(['PSNR_{ampl},  \sigma = ', num2str(sigma0)]),  grid on, xlabel('Iter. num.'), ylabel('dB')
  pause(.1)
end 

 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%%%%%%%%%% Phase unwrapping %%%%%%%%%%%%%%%
 XX=Iterative_CDBM3D_hat;  PUMA_SCRIPT;
 ITER_BM3D_toc=toc;
 %%%%%%%%%% Accuracy results for ITERATIVE CD-BM3D %%%%%%%%%%%%%%%%%%%%%%
[PSNR_PHASE, RMSE_abs_phase, ISNR_PHASE,ISNR_exp, PSNR_a_PHASE, NELP_PHASE, PSNR_AMPL, ISNR_AMPL]=Accuracy_Criteria_Complex(Z,varphi,B,Iterative_CDBM3D_hat,unwrap_phase);



PSNR_PHASE_ITER_CDBM3D_set(s_lambda3D,s_sigma)=PSNR_PHASE(end)
RMSE_abs_phase_ITER_CDBM3D_set(s_lambda3D,s_sigma)=RMSE_abs_phase(end); %#ok<*SAGROW>
ISNR_PHASE_ITER_CDBM3D_set(s_lambda3D,s_sigma)=ISNR_PHASE;
ISNR_exp_ITER_CDBM3D_set(s_lambda3D,s_sigma)=ISNR_exp;
PSNR_a_PHASE_ITER_CDBM3D_set(s_lambda3D,s_sigma)=PSNR_a_PHASE;
NELP_PHASE_ITER_CDBM3D_set(s_lambda3D,s_sigma)=NELP_PHASE;
PSNR_AMPL_ITER_CDBM3D_set(s_lambda3D,s_sigma)=PSNR_AMPL;
ISNR_AMPL_ITER_CDBM3D_set(s_lambda3D,s_sigma)=ISNR_AMPL;
ttime_ITER_CDBM3D_set(s_lambda3D,s_sigma)=ITER_BM3D_toc;
%%%%%%%%%%%%%%%%%

if if_figure_on==1
figure, subplot(2,2,1), imshow((unwrap_phase- quadratic_phase),[]), title(['ITER CDBM3D ABS PHASE, RMSE = ', num2str(RMSE_abs_phase(end),3), ' dB']),...
    subplot(2,2,2), imshow(abs(Iterative_CDBM3D_hat),[]), title(['AMPL, PSNR = ', num2str(PSNR_AMPL(end),3), ' dB']),...
    subplot(2,2,3), imshow((wrap(angle(Iterative_CDBM3D_hat)- quadratic_phase)),[]), title(['WRAP PHASE, PSNR = ', num2str(PSNR_PHASE(end),3), ' dB']),...
    subplot(2,2,4), imshow(wrap((angle(Iterative_CDBM3D_hat)- quadratic_phase-varphi)),[]), title(['WRAP PHASE ERRORS, \sigma= ',num2str(sigma0) ])

end


end  % end for s_lambda3D (Iterative_CDBM3D algorithm)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Windowed Fourier Transform (WFT, KEMAO) ALGORITHM %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if if_KEMAO==1
    display('WFT')

tic
  % g=wft2f('wff',Z,10,-3,0.1,3,10,-3,0.1,3,4*sigma/255);
  g=wft2fw('wff',Z,4,-pi,0.1,pi,4,-pi,0.1,pi,3*sigma/255);
 

%% g.filtered is a complex-valued estimate of the wavefield %%%%%%%%%%%
  
  %%%%%%%%%% 'PUMA' phase unwrapping for WFT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if if_unwrapping==1
potential.quantized = 'no';
potential.threshold = 0.5;
verbose='no';
unwrap_phase_wft  = puma_ho(angle(g.filtered),0.5, 'potential',potential,'verbose',verbose); 

else
unwrap_phase_wft = angle(g.filtered)*0;    %% no phase unwrapping
end
ttime_WFT(s_sigma)=toc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% Accuracy results for WFT %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[PSNR_PHASE_wft, RMSE_abs_phase_wft, ISNR_PHASE_wft,ISNR_exp_wft, PSNR_a_PHASE_wft, NELP_PHASE_wft, PSNR_AMPL_wft, ISNR_AMPL_wft]=...
    Accuracy_Criteria_Complex(Z,varphi,B,g.filtered, unwrap_phase_wft);
   
PSNR_PHASE_wft_set(s_sigma)=PSNR_PHASE_wft;
RMSE_abs_phase_wft_set(s_sigma)=RMSE_abs_phase_wft;
ISNR_PHASE_wft_set(s_sigma)=ISNR_PHASE_wft;
ISNR_exp_wft_set(s_sigma)=ISNR_exp_wft;
PSNR_a_PHASE_wft_set(s_sigma)=PSNR_a_PHASE_wft;
NELP_PHASE_wft_set(s_sigma)=NELP_PHASE_wft;
PSNR_AMPL_wft_set(s_sigma)=PSNR_AMPL_wft; 
ISNR_AMPL_wft_set(s_sigma)=ISNR_AMPL_wft;



if if_figure_on==1
figure, subplot(2,2,1), imshow((unwrap_phase_wft- quadratic_phase),[]), title(['WFT ABS PHASE, RMSE = ', num2str(RMSE_abs_phase_wft,3), ' dB']),...
    subplot(2,2,2), imshow(abs(g.filtered),[]), title(['AMPL, PSNR = ', num2str(PSNR_AMPL_wft,3), ' dB']),...
    subplot(2,2,3), imshow((wrap(angle(g.filtered)- quadratic_phase)),[]), title(['WRAP PHASE, PSNR = ', num2str(PSNR_PHASE_wft,3), ' dB']),...
    subplot(2,2,4), imshow(wrap((angle(g.filtered)- quadratic_phase-varphi)),[]), title(['WRAP PHASE ERRORS, \sigma= ',num2str(sigma0) ])

end

   
   
   
end  %% end for WFT algorithm
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% SpInPHASE ALGORITHM %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if if_SpInPHASE==1
tic
disp('SpInPHASE')

%  Parameters for SpInPHASE
param.K = 256;
param.T = 500;
param.sigma = sigma/255;
param.patchnum = 64;
param.lamda = 0.11;      %when sigma = 0.9  lamda = 0.15  when sigma < 0.9  lamda = 0.11
param.patsize = 10;
param.IstrainDic = 1;
param.IsUnwrap = if_unwrapping;


[SpInPHASE_hat,SpInPHASE_hat_ampl,output,alpha, unwrap_phase_SpInPHASE] = SpInPHASE_ampl(Z, param);
ttime_SpInPHASE(s_sigma)=toc;
if if_unwrapping==0
 unwrap_phase_SpInPHASE=angle(SpInPHASE_hat);

    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% Accuracy results for SpInPHASE %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[PSNR_PHASE_SpInPHASE,RMSE_abs_phase_SpInPHASE,ISNR_PHASE_SpInPHASE, ISNR_exp_SpInPHASE,PSNR_a_PHASE_SpInPHASE, NELP_PHASE_SpInPHASE,PSNR_AMPL_SpInPHASE, ISNR_AMPL_SpInPHASE]=...
Accuracy_Criteria_Complex(Z,varphi,B,SpInPHASE_hat_ampl.*exp(1j*SpInPHASE_hat),unwrap_phase_SpInPHASE);


PSNR_PHASE_SpInPHASE_set(s_sigma)=PSNR_PHASE_SpInPHASE ;
RMSE_abs_phase_SpInPHASE_set(s_sigma)=RMSE_abs_phase_SpInPHASE;
ISNR_PHASE_SpInPHASE_set(s_sigma)=ISNR_PHASE_SpInPHASE;
ISNR_exp_SpInPHASE_set(s_sigma)=ISNR_exp_SpInPHASE;
PSNR_a_PHASE_SpInPHASE_set(s_sigma)=PSNR_a_PHASE_SpInPHASE;
NELP_PHASE_SpInPHASE_set(s_sigma)=NELP_PHASE_SpInPHASE;
PSNR_AMPL_SpInPHASE_set(s_sigma)=PSNR_AMPL_SpInPHASE; 

ISNR_AMPL_SpInPHASE_set(s_sigma)=ISNR_AMPL_SpInPHASE;


 

if if_figure_on==1
figure, subplot(2,2,1), imshow((unwrap_phase_SpInPHASE- quadratic_phase),[]), title(['SpInPHASE ABS PHASE, RMSE = ', num2str(RMSE_abs_phase_SpInPHASE,3), ' dB']),...
    subplot(2,2,2), imshow(SpInPHASE_hat_ampl,[]), title(['AMPL, PSNR = ', num2str(PSNR_AMPL_SpInPHASE,3), ' dB']),...
    subplot(2,2,3), imshow(wrap(SpInPHASE_hat- quadratic_phase),[]), title(['WRAP PHASE, PSNR = ', num2str(PSNR_PHASE_SpInPHASE,3), ' dB']),...
    subplot(2,2,4), imshow(wrap((SpInPHASE_hat- quadratic_phase-varphi)),[]), title(['WRAP PHASE ERRORS, \sigma= ',num2str(sigma0) ])

end


 
end  %% end for SpInPhase


 end  %% end for s_sigma
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% TABLE OF RESULTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if 1
format short
disp(' ALGORITHM COMPARISON RESULTS')
if 1&(if_SpInPHASE==1)&(if_KEMAO==1)&(if_ITERATIVE_SPAR==1)
%%%%%%%%%%%%% TABLE of PHASE RESULTS %%%%%%%%%%%%%%
A_phase=[sigma_set' SNR' PSNR_PHASE_CDBM3D_set' PSNR_PHASE_ITER_CDBM3D_set(end,:)' PSNR_PHASE_SPAR_set' PSNR_PHASE_SpInPHASE_set'  PSNR_PHASE_wft_set' RMSE_abs_phase_CDBM3D_set' RMSE_abs_phase_ITER_CDBM3D_set' RMSE_abs_phase_SPAR_set' RMSE_abs_phase_SpInPHASE_set'  RMSE_abs_phase_wft_set'];
T_phase = array2table(A_phase,'VariableNames',{'sigma' 'SNR' 'PSNR_BM1' 'PSNR_BM2' 'PSNR_SPAR' 'PSNR_Sp' 'PSNR_WFT' 'RMSE_abs_BM1' 'RMSE_abs_BM2' 'RMSE_abs_SPAR' 'RMSE_abs_Sp' 'RMSE_abs_WFT'})

if if_unwrapping==0
disp('No unwrapping , RMSE calculated for abs phase is wrong')
end

%%%%%%%%%%%%% TABLE of AMPLITUDE RESULTS %%%%%%%%%%%%%%

A_ampl=[sigma_set' SNR'  PSNR_AMPL_CDBM3D_set' PSNR_AMPL_ITER_CDBM3D_set' PSNR_AMPL_SPAR_set' PSNR_AMPL_SpInPHASE_set' PSNR_AMPL_wft_set'];

T_ampl = array2table(A_ampl,'VariableNames',{'sigma' 'SNR' 'PSNR_BM1' 'PSNR_BM2' 'PSNR_SPAR' 'PSNR_Sp' 'PSNR_wft' })

A_time=[sigma_set' ttime_CDBM3D_set' ttime_ITER_CDBM3D_set' ttime_SPAR' ttime_SpInPHASE' ttime_WFT'];

T_time = array2table(A_time,'VariableNames',{'sigma' 'time_BM1' 'time_ITER_BM2' 'time_SPAR' 'time_Sp' 'time_WFT'})


elseif 1&(if_SpInPHASE==0)&(if_KEMAO==0)&(if_ITERATIVE_SPAR==1)
%%%%%%%%%%%%% TABLE of PHASE RESULTS %%%%%%%%%%%%%%
A_phase=[sigma_set' SNR' PSNR_PHASE_CDBM3D_set' PSNR_PHASE_ITER_CDBM3D_set(end,:)' PSNR_PHASE_SPAR_set'   RMSE_abs_phase_CDBM3D_set' RMSE_abs_phase_ITER_CDBM3D_set' RMSE_abs_phase_SPAR_set' ];
T_phase = array2table(A_phase,'VariableNames',{'sigma' 'SNR' 'PSNR_BM1' 'PSNR_BM2' 'PSNR_SPAR'  'RMSE_abs_BM1' 'RMSE_abs_BM2' 'RMSE_abs_SPAR' })

if if_unwrapping==0
disp('No unwrapping , RMSE calculated for abs phase is wrong')
end

%%%%%%%%%%%%% TABLE of AMPLITUDE RESULTS %%%%%%%%%%%%%%

A_ampl=[sigma_set' SNR'  PSNR_AMPL_CDBM3D_set' PSNR_AMPL_ITER_CDBM3D_set' PSNR_AMPL_SPAR_set'];

T_ampl = array2table(A_ampl,'VariableNames',{'sigma' 'SNR' 'PSNR_BM1' 'PSNR_BM2' 'PSNR_SPAR'  })


%%%%%%%%%
A_time=[sigma_set' ttime_CDBM3D_set' ttime_ITER_CDBM3D_set' ttime_SPAR' ];

T_time = array2table(A_time,'VariableNames',{'sigma' 'time_BM1' 'time_ITER_BM2' 'time_SPAR' })


%%%%%%%%%%%%%%%
elseif 1&(if_SpInPHASE==1)&(if_KEMAO==1)&(if_ITERATIVE_SPAR==0)
%%%%%%%%%%%%% TABLE of PHASE RESULTS %%%%%%%%%%%%%%
A_phase=[sigma_set' SNR' PSNR_PHASE_CDBM3D_set' PSNR_PHASE_ITER_CDBM3D_set(end,:)' PSNR_PHASE_SpInPHASE_set'  PSNR_PHASE_wft_set' RMSE_abs_phase_CDBM3D_set' RMSE_abs_phase_ITER_CDBM3D_set'  RMSE_abs_phase_SpInPHASE_set'  RMSE_abs_phase_wft_set'];
T_phase = array2table(A_phase,'VariableNames',{'sigma' 'SNR' 'PSNR_BM1' 'PSNR_BM2' 'PSNR_Sp' 'PSNR_WFT' 'RMSE_abs_BM1' 'RMSE_abs_BM2' 'RMSE_abs_Sp' 'RMSE_abs_WFT'})

if if_unwrapping==0
disp('No unwrapping , RMSE calculated for abs phase is wrong')
end

%%%%%%%%%%%%% TABLE of AMPLITUDE RESULTS %%%%%%%%%%%%%%

A_ampl=[sigma_set' SNR'  PSNR_AMPL_CDBM3D_set' PSNR_AMPL_ITER_CDBM3D_set' PSNR_AMPL_SpInPHASE_set' PSNR_AMPL_wft_set'];

T_ampl = array2table(A_ampl,'VariableNames',{'sigma' 'SNR' 'PSNR_BM1' 'PSNR_BM2' 'PSNR_Sp' 'PSNR_wft' })

A_time=[sigma_set' ttime_CDBM3D_set' ttime_ITER_CDBM3D_set' ttime_SpInPHASE' ttime_WFT'];

T_time = array2table(A_time,'VariableNames',{'sigma' 'time_BM1' 'time_ITER_BM2' 'time_Sp' 'time_WFT'})

%%%%%%%%%%%%%%%%
else  %%1&(if_SpInPHASE==0)&(if_KEMAO==0)&(if_ITERATIVE_SPAR==1)  
    %%%%%%%%%%%%% TABLE of PHASE RESULTS %%%%%%%%%%%%%%

A_phase=[sigma_set' SNR' PSNR_PHASE_CDBM3D_set' PSNR_PHASE_ITER_CDBM3D_set(end,:)' RMSE_abs_phase_CDBM3D_set' RMSE_abs_phase_ITER_CDBM3D_set(end,:)' ];

T_phase = array2table(A_phase,'VariableNames',{'sigma' 'SNR' 'PSNR_BM1' 'PSNR_BM2' 'RMSE_abs_BM1' 'RMSE_abs_BM2' })



if if_unwrapping==0
disp('No unwrapping , RMSE calculated for abs phase is wrong')
end

%%%%%%%%%%%%% TABLE of AMPLITUDE RESULTS %%%%%%%%%%%%%%

A_ampl=[sigma_set' SNR' PSNR_AMPL_CDBM3D_set' PSNR_AMPL_ITER_CDBM3D_set'];

T_ampl = array2table(A_ampl,'VariableNames',{'sigma' 'SNR' 'PSNR_BM1' 'PSNR_BM2' })


A_time=[sigma_set' ttime_CDBM3D_set' ttime_ITER_CDBM3D_set' ];

T_time = array2table(A_time,'VariableNames',{'sigma' 'time_BM1' 'time_ITER_BM2' })

end
end


