%% Iterative CDBM3D algorithm 
%% Input: Z-complex-valued observations,u-Initial guess,param-lisr of parameters,B-true amplitude,varphi-true phase
%% Output: Iterative_CDBM3D_hat - estimate, nnzTransfElements - statistics of the number of used patches,
%% PSNR_PHASE -criterion for phase ,PSNR_AMPL- criterion for amplitude

%% LIST of REQUIRED PARAMETERS

% param.nm1 = 8;  %% N1 x N1 is the block size used for the hard-thresholding (HT) filtering
% param.Nstep =3; %% sliding step to process every next reference block
% param.nm2 = floor(nm1*nm1)/4; %% maximum number of similar blocks (maximum size of the 3rd dimension of a 3D array)
% param.Ns = 39;  %% length of the side of the search neighborhood for full-search block-matching (BM), must be odd
%
% param.lambda3D=2;        %% parameter for the thresholding
% param.lambda_conf_int=3; %% parameter for the two sides symmetric confidence interval for grouping
% param.lambda_conf_int_factor=5; %% parameter for the upper bound in the two sides non-symmetric confidence interval for grouping
% param.doBlockMatching=true;  %% true means that the new grouping is produced, false means that the previous grouping is used
% param.bm2D3Dhosvd_index3D=1;  %% 1 stays for high-order 3D SVD, otherwise 2D SVD
% param.sigma = sigma;          %% STD of the additive Gaussian noise
% param.lambda2D       =  0.0;   %% threshold parameter for the coarse initial denoising used in the d-distance measure
% param.originalphase = varphi;
%% lower bound of confidence interval;
% param.lambda_conf(1)=max(0,(param.nm1^2-param.lambda_conf_int*sqrt(2*param.nm1^2))*param.sigma^2*2/255^2);
% upper bound of confidence interval;
% param.lambda_conf(2)= (param.nm1^2+param.lambda_conf_int*sqrt(2*param.nm1^2))*param.sigma^2*2/255^2*param.lambda_conf_int_factor;
% param.iterNumber=6;
% param.gamma_10=1.5;
% 
% param.a1=.1/2/2/2;
% param.a2=.01/2/2/2;

%% Author: Vladimir Katkovnik, Tampere, March 2016


function [Iterative_CDBM3D_hat,PSNR_PHASE,PSNR_AMPL]  = ITERATIVE_CDBM3Dp(Z,u,param,B,varphi)
sigma=param.sigma;
Nr=param.iterNumber;
alpha_0=param.alpha;
a1=param.a1;
a2=param.a2;
[M,N]=size(Z);
lambda3D_0=param.lambda3D;
%% Iterations 
for rr=1:Nr
 %alpha_var=alpha_0/(1+a1*(rr-1));
 param.alpha=param.alpha*a1;
 alpha_var=param.alpha;
% betta_var=1/(1+a2*(rr-1));
  v=Z*alpha_var+u*(1-alpha_var);
  %%  param.lambda3D=param.lambda3D/(1+a2*(rr-1));
%% temp=function_stdEst2D([real(u(:)) imag(u(:))],0);
param.lambda3D=param.lambda3D*a2;
%% param.sigma
 
if rr>300   %% to stop update of grouping
    param.doBlockMatching=false;
end
%% param.sigma=temp*255;

[u]  = CDBM3Dp(v, param);
%% [u]  = CDBM3D_D(v, param);

%% figure, imshow(angle(u),[]), title('angle(u)')

if nargin>3
vv1=sum((abs(u(:))-B(:)).^2);  
vv1=norm(wrap(angle(u) - varphi),'fro')^2; 
PSNR_PHASE(rr)=10*log10(4*M*N*pi^2/vv1);
PSNR_AMPL(rr)=10*log10(M*N*max(abs(B(:)).^2)/vv1);
else
 PSNR_PHASE=0; PSNR_AMPL=0;   
end
end
%% final estimate
Iterative_CDBM3D_hat=u;
end

