function [image_esti,image_ampl,output,alpha, phi_hat_unwrap] = SpInPHASE_ampl(Z, param, D)
%%            denoise based on the complex value dictionary learning
%%%%%%%%%%%%%%%%%%%%%%%%%%%%input%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Z                            complex number matrix(with noise)
% param                       a structure of parameters
%  param.patsize              width or height of the patch(usually width=height)
%                             defaul:12
%  param.IstrainDic           train the dictionary or not(1---train the dictionary 0---do not train the dicionary)
%                             defaul:1
%  param.sigma                the noise variance of the real part(or imaginary part): should be a matrix or a real number
%                             defaul: estimate automatically
%  param.mask                 the mask: should be a matrix with 0 and 1
%                             defaul: estimate automatically
%  param.IsUnwrap             perform the unwrapping or not(1---unwrapping 0---do not unwrapping)
%                             defaul: 0
%  param.IsOutputC            output the complex number or the estimated phase(1---complex number 0---estimated phase)
%                             defaul: 0
%  param.IsSubMean            Substract the mean or not(1---substract mean of every patch 0---do not substract mean of every patch)
%                             defaul: 0
%  param.originalphase        the original unwrapped phase for us to  compute the Mean Squre Error and ISNR 
%                             no defaul
%%%%%%%%%%%%%%%%%%%%%%%%parameters for dictionary learning%%%%%%%%%%%
%  param.K                    number of atoms in the trained dictionary
%                             defaul: 256
%  param.T                    number of iterations in the trained dictionary
%                             defaul: 500
%  param.patchnum             number of patches use in one ieration in the trained dictionary
%                             defaul: 64
%  param.lamda                parameter in SUNSAL to solve the 
%                             defaul: 1.8
%  param.err                  tolerant error in the SUNSAL algorithm
%                             defaul: 1e-2
%  param.dicname              the default dictionary name
%                             defaul: 'dic%dX%d.mat'  %d is the param.patsize
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%optional input dictionary%%%%%%%%%%%%%%%%%%
%  D                          the dictionary learned before hand
%%%%%%%%%%%%%%%%%%%%%%%%%%%%output%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  image_esti                   denoise complex value matrix
%  output                       learned dictionary
%  alpha                        coding of the patches based on the dictionary
%  phi_hat_unwrap               unwrapped surface if param.IsUnwrap==1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Z is real if it is the input wrapped noisy phase
if imag(Z)==0
    Z = exp(sqrt(-1)*Z);
end

if isfield(param, 'patsize')
    patsize = param.patsize;
else
    patsize = 12;
end

if isfield(param, 'IstrainDic')
    IstrainDic = param.IstrainDic;
else
    IstrainDic = 1;
end

if isfield(param, 'sigma')
    sigmaO = max(param.sigma,0.1);
else
    sigmaO = max(function_stdEst2D_phase(real(Z),1),0.1); 
end
if isfield(param, 'mask')
    mask = param.mask;
else
    mask = ones(size(Z));
end

if size(sigmaO) == size(Z)
    Z = Z./sigmaO;
    sigma = sqrt(2);
else
    sigma = sqrt(2)*sigmaO;
end

if isfield(param, 'IsUnwrap') 
    IsUnwrap = param.IsUnwrap;
else
    IsUnwrap = 0; 
end
if isfield(param, 'IsOutputC')
    IsOutputC = param.IsOutputC;
else
    IsOutputC = 0; 
end
if isfield(param, 'IsSubMean')
    IsSubMean = param.IsSubMean;
else
    IsSubMean = 0; 
end
%% initial parameters for dictionary learning
if isfield(param, 'K')
    paramDL.K = param.K;
else
    paramDL.K = 256; 
end
if isfield(param, 'T')
    paramDL.T = param.T;
else
    paramDL.T = 500; 
end
if isfield(param, 'patchnum')
    paramDL.patchnum = param.patchnum;
else
    paramDL.patchnum = 64; 
end
if isfield(param, 'lamda')
    paramDL.lamda = param.lamda;
else
    paramDL.lamda = 0.11; 
end
if isfield(param, 'err')
    paramDL.err = param.err;
else
    paramDL.err = 1e-3; 
end
if isfield(param, 'Dinitial')
    paramDL.Dinitial = param.Dinitial;
end

if isfield(param, 'dicname')
    dicfile = param.dicname;
else
    if IsSubMean==1
        dicfile = sprintf('sdic%dX%d.mat',patsize,patsize);
    else
        dicfile = sprintf('dic%dX%d.mat',patsize,patsize);
    end
end

if isfield(param, 'originalphase')
    originalphase = param.originalphase;
    haveoriginal = 1;
else
    haveoriginal = 0; 
end

if isfield(param, 'L')
    paramOMP.L = param.L;
end

isinputdic = 0;
if(nargin==3)
    if size(D,1)~=patsize*patsize
        error(['The size of the dictionary do not match the patsize']);
    else
        isinputdic = 1;
    end
end


X = im2col(Z,[patsize patsize],'sliding');
maskC = im2col(mask,[patsize patsize],'sliding');
indexm = find(sum(abs(maskC))~=0);
clear maskC;
Xmask = X(:,indexm);
if IsSubMean == 1
    meancolx = angle(sum(Xmask));
    Xmask = Xmask.*(ones(size(Xmask,1),1)*exp(-1i*meancolx));
%     meancolx = mean(Xmask);
%     Xmask=Xmask-repmat(meancolx,[size(Xmask,1) 1]);
end

%train the dictionary from the noisy image
if isinputdic==0
    if IstrainDic==1
        %     tic
        [D] = DicLearningM(Xmask,paramDL);
        %     toc
        
        %     t=toc;
        %     fprintf('time of computation for Dictionary Learning: %f\n',t);
    elseif IstrainDic==0
        if ~exist(dicfile,'file')
            error(['Can not find file ''' dicfile '''']);
        else
            load(dicfile);
        end
    end
end

output = D;

fprintf('Evaluating cost function...\n');
paramOMP.err = chi2inv(0.96,size(Xmask,1)*2)*sigma^2/2;
alpha=OMP_C(output,Xmask,paramOMP);



xx_hatmask = output*alpha;
if IsSubMean == 1
    xx_hatmask=xx_hatmask.*(ones(size(xx_hatmask,1),1)*exp(1i*meancolx));
%     xx_hatmask=xx_hatmask+repmat(meancolx,[size(xx_hatmask,1) 1]);
end
% xx_hat=xx_hat+repmat(meancolx,[size(xx_hat,1) 1]);

X(:,indexm) = xx_hatmask;

blockx = patsize;
blocky = patsize;
final_numestimate = zeros(size(Z));
final_extentestimate = zeros(size(Z));
for indexi = 1:blocky
    for indexj = 1:blockx
        tempesti = reshape(X((indexi-1)*blockx+indexj,:),size(Z)-[blockx,blocky]+1);
        numestimate = zeros(size(Z));
        extentestimate = zeros(size(Z));
        extentestimate(1:size(tempesti,1),1:size(tempesti,2)) = tempesti;
        numestimate(1:size(tempesti,1),1:size(tempesti,2)) = 1;
        
        extentestimate = circshift(extentestimate, [indexj,indexi]-1);
        numestimate = circshift(numestimate, [indexj,indexi]-1);
        
        final_numestimate = final_numestimate+numestimate;
        final_extentestimate = final_extentestimate+extentestimate;
    end
end
image_esti_c = final_extentestimate./final_numestimate;
%%
image_esti = angle(image_esti_c);

%%%%%%% my correction for  amplitude estimation %%%%%%%%%
image_ampl=abs(image_esti_c);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
phi_hat_unwrap = zeros(size(image_esti));
if IsUnwrap==1
    phi_hat_unwrap = puma_ho(image_esti ,0.5, 'verbose','no' );
end

if haveoriginal==1
    wraperr_norm = norm(wrap(image_esti - originalphase),'fro')^2;
    RMSE_SpInPHASE = sqrt(wraperr_norm/M/N);
    PSNR_SpInPHASE = 10*log10(4*M*N*pi^2/wraperr_norm);
    fprintf('\n\n Performance indices\n')
    fprintf('RMSE (SpInPHASE) = %2.3f\n', RMSE_SpInPHASE)
    fprintf('PSNR (SpInPHASE) = %2.3f\n', PSNR_SpInPHASE)
end
if IsOutputC==1
    image_esti = image_esti_c;
end
